import time

import machine

import json
import simpleHttpServer


class ConfigHttpServer(simpleHttpServer.SimpleHttpServer):

    def __init__(self, max_concurrent_requests):
        super(ConfigHttpServer, self).__init__(self.route_client_request, max_concurrent_requests)
        self.form_submission_controller = FormSubmissionController()
        self.form_display_controller = FormDisplayController()

    def route_client_request(self, req, resp):

        if req.params.get('ssid'):
            self.form_submission_controller.handle_form_submission(req, resp)
        else:
            self.form_display_controller.display_form(resp)


class FormDisplayController:

    def display_form(self, resp):
        resp.send(self.default_config_web_page())

    @staticmethod
    def default_config_web_page():
        return """
    <!DOCTYPE html>
    <html>
    <body>
    
    <h2>Configure Network</h2>
    <p>enter your network ssid and password</p>
    
    <form action="">
    ssid:<br>
    <input type="text" name="ssid">
    <br>
    password:<br>
    <input type="text" name="password">
    <br><br>
    <input type="submit">
    </form>
    
    </body>
    </html>
        """


class FormSubmissionController:
    def handle_form_submission(self, req, resp):
        station_id = req.params.get('ssid')
        password = req.params.get('password')
        config = {"ssid": station_id, "password": password}
        self.write_config(config)
        resp.send(self.rebooting_web_page(station_id))
        self.reboot_device()

    @staticmethod
    def reboot_device():
        time.sleep(2)
        machine.reset()

    @staticmethod
    def write_config(config):
        f = open("config.json", 'w')
        f.write(json.dumps(config))
        f.close()

    @staticmethod
    def rebooting_web_page(station_id):
        return "rebooting to connect to " + station_id



import machine


class ConnectStatus:
    DISCONNECTED = 0
    CONNECTING = 1
    CONNECTED = 2
    FAILED = 3


class NetworkStatusNotifier:

    def __init__(self):
        self.status = ConnectStatus.DISCONNECTED

    def setConnected(self):
        self.status = ConnectStatus.CONNECTED

    def setConnecting(self):
        self.status = ConnectStatus.CONNECTING

    def setFailed(self):
        self.status = ConnectStatus.FAILED

    def setDisconnected(self):
        self.status = ConnectStatus.DISCONNECTED


class BuiltInLedNetworkStatusNotifier(NetworkStatusNotifier):

    def __init__(self):
        super(BuiltInLedNetworkStatusNotifier, self).__init__()
        self.led = machine.Pin(2, machine.Pin.OUT)
        self.led.off()
        self.ledTimer = machine.Timer(0)
        self.setDisconnected()

    def setConnecting(self):
        super(BuiltInLedNetworkStatusNotifier, self).setConnecting()
        self.ledTimer.deinit()
        self.led.on()

    def setConnected(self):
        super(BuiltInLedNetworkStatusNotifier, self).setConnected()
        self.ledTimer.deinit()
        self.led.off()

    def setFailed(self):
        super(BuiltInLedNetworkStatusNotifier, self).setFailed()
        self.ledTimer.init(period=100, callback=self.toggleLedCallback)

    def setDisconnected(self):
        super(BuiltInLedNetworkStatusNotifier, self).setDisconnected()
        self.ledTimer.init(period=500, callback=self.toggleLedCallback)

    # noinspection PyUnusedLocal
    def toggleLedCallback(self, timer):
        if self.led.value():
            self.led.off()
        else:
            self.led.on()



import socket
import gc
import time
import network


class HttpLocalHostListener:

    def __init__(self, numberOfListeners):
        sta_if = network.WLAN(network.STA_IF)
        sta_if.active(False)
        ap_if = network.WLAN(network.AP_IF)
        ap_if.active(True)
        gc.collect()
        time.sleep(2)
        self.listener_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)

        self.listener_socket.bind(('', 80))  # TODO: retry logic?...OSError:112
        self.listener_socket.listen(numberOfListeners)



_hex_byte_cache = None


class HttpRequest:

    def __init__(self, client_socket):
        self.client_socket = client_socket
        cl_file = client_socket.makefile('rwb', 0)  # TODO: does the buffer really need to be writable?
        self.raw_request_headers = self.get_raw_request_headers(cl_file)
        self.params = {}
        self.path = ''
        self.parse_path()

    @staticmethod
    def get_raw_request_headers(cl_file):
        request_headers = []
        while True:
            line = cl_file.readline()
            request_headers.append(line.decode('utf-8'))
            if not line or line == b'\r\n':
                break
        return request_headers

    def parse_path(self):
        url = self.raw_request_headers[0].split(' ')[1]
        path_and_raw_params = url.split('?')
        self.path = path_and_raw_params[0]

        if len(path_and_raw_params) == 1:
            return
        param_pairs = path_and_raw_params[1].split('&')

        for param_pair in param_pairs:
            split = param_pair.split('=')
            self.params.__setitem__(unquote(split[0]).decode('utf-8'), unquote(split[1]).decode('utf-8'))


class HttpResponse:

    def __init__(self, client_socket):
        self.client_socket = client_socket

    def send(self, content):
        self.client_socket.send(content)
        self.client_socket.close()


def unquote(string):
    """unquote('abc%20def') -> b'abc def'."""
    global _hex_byte_cache

    # Note: strings are encoded as UTF-8. This is only an issue if it contains
    # unescaped non-ASCII characters, which URIs should not.
    if not string:
        return b''

    if isinstance(string, str):
        string = string.replace('+', ' ')
        string = string.encode('utf-8')

    bits = string.split(b'%')
    if len(bits) == 1:
        return string

    res = [bits[0]]
    append = res.append

    # Build cache for hex to char mapping on-the-fly only for codes
    # that are actually used
    if _hex_byte_cache is None:
        _hex_byte_cache = {}

    for item in bits[1:]:
        try:
            code = item[:2]
            char = _hex_byte_cache.get(code)
            if char is None:
                char = _hex_byte_cache[code] = bytes([int(code, 16)])
            append(char)
            append(item[2:])
        except KeyError:
            append(b'%')
            append(item)

    return b''.join(res)




from distutils.core import setup


def read(file_relative):
    file = file_relative
    with open(str(file)) as f:
        return f.read()


setup(
    name='esp32_net_config',
    packages=[''],
    version='0.2.7',
    license='MIT',
    description='module to allow configuration of netword ssid and password through local access point',
    long_description=read('README.rst'),
    author='Todd Flanders',
    author_email='toddfbass@gmail.com',
    url='https://github.com/tflander/esp32-machine-emulator',
    download_url='https://github.com/tflander/esp32-machine-emulator/archive/v_01.tar.gz',  # I explain this later on
    keywords=['ESP32', "MicroPython"],
    install_requires=[],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: Implementation :: MicroPython'
    ],
)



import json


class Config:

    def __init__(self, ssid, password):
        self.ssid = ssid
        self.password = password
        self.exists = True

    def asJson(self):
        return json.dumps({"ssid": self.ssid, "password": self.password})

    @classmethod
    def read(cls, filename):
        try:
            f = open(filename)
        except OSError:
            c = Config('', '')
            c.exists = False
            return c
        configDictionary = json.loads(f.read())
        f.close()
        return Config(configDictionary.get('ssid'), configDictionary.get('password'))

    def write(self, filename):
        f = open(filename, "w")
        f.write(self.asJson())
        f.close()



import http_support
import httpLocalHostListener


class SimpleHttpServer:

    def __init__(self, client_request_router, max_concurrent_requests):
        self.max_concurrent_requests = max_concurrent_requests
        self.client_request_router = client_request_router
        self.listener = None

    def init(self):
        self.listener = self.create_listener()

    def dispatch_client_requests(self):
        client_socket, addr = self.listener.listener_socket.accept()
        req = http_support.HttpRequest(client_socket)
        resp = http_support.HttpResponse(client_socket)
        self.client_request_router(req, resp)

    def create_listener(self):
        return httpLocalHostListener.HttpLocalHostListener(self.max_concurrent_requests)



import network
import socket
import configHttpServer


class NetConfigAccessPoint:

    def __init__(self, localSsid, max_concurrent_requests=5):
        self.max_concurrent_requests = max_concurrent_requests
        self.ssid = localSsid

    def start(self):
        self.create_access_point()  # address is always http://192.168.4.1
        http_server = configHttpServer.ConfigHttpServer(self.max_concurrent_requests)
        http_server.init()

        while True:
            http_server.dispatch_client_requests()

    def create_access_point(self):
        ap_if = network.WLAN(network.AP_IF)
        ap_if.active(True)
        ap_if.config(essid=self.ssid)  # fails if we tried to go to st mode..
        addr = socket.getaddrinfo('0.0.0.0', 80)[0][-1]
        return addr, ap_if.ifconfig()



import gc
import socket
import time

import network

import config
import httpLocalHostListener
import netConfigAccessPoint
import networkStatusNotifier


def connect_network_or_go_into_config_mode(ssid_for_ap_connection, network_config_notifier=None):
    net_notifier = get_net_notifier(network_config_notifier)
    existing_config = config.Config.read('config.json')
    if existing_config.exists:
        print('connecting to ' + existing_config.ssid + '...')
        local_address, if_config = wifi_connect(existing_config.ssid, existing_config.password, net_notifier)
        print(local_address)
        if not local_address:
            print('connect failed')
            net_notifier.setFailed()
            start_ap_config_server(ssid_for_ap_connection)
    else:
        net_notifier.setDisconnected()
        start_ap_config_server(ssid_for_ap_connection)


def get_net_notifier(network_config_notifier):
    net_notifier = network_config_notifier
    if not net_notifier:
        net_notifier = networkStatusNotifier.BuiltInLedNetworkStatusNotifier()
    return net_notifier


def wifi_connect(remote_sid, password, net_notifier):
    sta_if = network.WLAN(network.STA_IF)
    sta_if.active(True)
    sta_if.connect(remote_sid, password)
    net_notifier.setConnecting()
    if not wait_for_network(sta_if, retries_for_timeout=80):
        net_notifier.setFailed()
        print("giving up on network " + remote_sid)
        sta_if.active(False)
        return None, None
    net_notifier.setConnected()

    addr = socket.getaddrinfo('0.0.0.0', 80)[0][-1]
    return addr, sta_if.ifconfig()


def wait_for_network(sta_if, retries_for_timeout):
    count = 0
    while not sta_if.isconnected():
        time.sleep_ms(200)
        count += 1
        if count > retries_for_timeout:
            return False
    return True


def start_ap_config_server(new_ap_ssid):
    gc.collect()
    print('connect to http://192.168.4.1 to configure')
    http_local_host_listener = create_http_local_host_listener(number_of_listeners=5)
    config_server = netConfigAccessPoint.NetConfigAccessPoint(new_ap_ssid, http_local_host_listener)
    config_server.start()


def create_http_local_host_listener(number_of_listeners):
    return httpLocalHostListener.HttpLocalHostListener(number_of_listeners)
